/* TPSRX200-2841 - replace UPB extension on MMX4x2-HDMI device if the one on the device is not right */

include("common_scripts_lib/legacy/tftp.js");
include("common_scripts_lib/legacy/package.js");
include("common_scripts_lib/legacy/uid.js");
include("common_scripts_lib/lw3/control_lw3.js");


function isUpbExtensionUpToDate(upbExtensionFile) {
    Object.defineProperty(this, 'Arrays', {value: Java.type("java.util.Arrays")});
    Object.defineProperty(this, 'IOUtils', {value: Java.type("org.apache.commons.io.IOUtils")});

    logger.info("Validating UPB Extension");
    try {
        var baos = new ByteArrayOutputStream();
        var argument = TftpOutputStreamArgument.newBuilder()
            .setOutputStream(baos)
            .setFileName(upbExtensionFile.getFileName())
            .setHostIp(deviceIp.getHostAddress())
            .setTftpMode(LwTFTP.OCTET_MODE)
            .build();

        upbExtensionFile.configTftpFile();

        var tftpGetFlow = new TftpGetFlow(argument);
        tftpGetFlow.execute();

        var upbExtContentOnDevice = tftpGetFlow.getDownloadedContent();
        var upbExtContentInPackage = IOUtils.toByteArray(packageFile.getFileHandler(upbExtensionFile.getFileNameInPackage()).getInputStream());

        if (!Arrays.equals(upbExtContentInPackage, upbExtContentOnDevice)) {
            logger.info("UPB extension needs to be updated");
            return false;
        } else {
            logger.info("UPB extension is up-to-date");
            return true;

        }
    } catch (e) {
        showRestartYourDevicePopup(deviceIp, UID_FVS_EEPROM_CONFIG, ownerWindow);
        throw new UpdateFailedException("Unable to validate UPB extension.", e);
    }
}

function replaceUpbExtension(upbExtensionFile, parentCheckpoint, checkpointWeight) { // TPSRX200-2841
    try {
        var checkpoint = parentCheckpoint.add("Replacing UPB Extension...", checkpointWeight);
        logger.info("Replacing UPB extension");
        uploadFileWithTftp(UPB_EXTENSION, checkpoint, 1.0);
    } catch (e) {
        showRestartYourDevicePopup(deviceIp, UID_FVS_EEPROM_CONFIG, ownerWindow);
        throw new UpdateFailedException("Failed to replace UPB extension.", e);
    }
}

function enterBootloadNoExtension() {  // TPSRX200-2841
    logger.info("Switching device into \"no-upb-extension\" bootloader mode...");
    try {
        var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);
        var bootloadMethod = new Method("bootload", new Path("/SYS"));

        new CallNoReturnMethod(connection, bootloadMethod, "1").execute();

        rootCheckpoint.waitMillisLong(1000);

    } catch (e) {
        throw new UpdateFailedException("Unable to start bootloader in no-upb-extension mode. (TPSRX200-2841)", e);
    }
}

function showRestartYourDevicePopup(deviceIp, uidEepromConfig, ownerWindow) {
    var contentText = "Please switch the device into application mode by unplugging and replugging the DC power adaptor. \n\nThen please start update process again.";

    var devicePopupArgument = DevicePopupArgument.newBuilder()
        .setTcpConnection(new TcpConnection(deviceIp, 6107, 1000, 1000, 1))
        .setUidEepromConfig(uidEepromConfig)
        .setOwnerWindow(ownerWindow)
        .setButtons(["OK"])
        .setContentText(contentText)
        .setHeaderText("Device is in inappropriate operation mode")
        .setIconType(DevicePopupArgument.IconType.ERROR)
        .build();

    var devicePopupShow = new DevicePopupShow(devicePopupArgument);

    devicePopupShow.execute();

    return devicePopupShow.getClickedButton();
}

function getPartNumber() { // TPSRX200-2841
    try {
        var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 2);
        var getPartNumberProperty = new GetProperty(connection, new Property("PartNumber", new Path("/")));
        getPartNumberProperty.execute();

        return getPartNumberProperty.getProperty().getCachedValue();
    } catch (e) {
        var uid = getUID(UID_FVS_EEPROM_CONFIG, rootCheckpoint, 0.0);
        return uid.getProductPartNumber();
    }
}

function getPackageVersionLW3AndUpb() { // TPSRX200-2841
    if (isLw3Capable()) {
        try {
            var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 2);
            var getPackageVersionProperty = new GetProperty(connection, new Property("PackageVersion", new Path("/MANAGEMENT/UID")));
            getPackageVersionProperty.execute();
            return getPackageVersionProperty.getProperty().getCachedValue();
        } catch (e) {
            return "v1.0.0b1"; // no package version property available
        }
    } else {
        return getPackageVersion(UID_FVS_EEPROM_CONFIG);
    }
}