include("common_scripts_lib/backup_and_restore/device_configuration_backup.js");
include("common_scripts_lib/backup_and_restore/preset_backup.js");
include("common_scripts_lib/backup_and_restore/miniweb_backup.js");
include("common_scripts_lib/progress/progressUtils.js");
include("common_scripts_lib/check/checkFwVersion.js");

function makeFullConfigurationBackup(minVersion, converterScript, parentCheckpoint, checkpointWeight, deviceIp, uidEepromConfig, ownerWindow) {
    var configBackupWeight = 0.4 * checkpointWeight;
    var presetBackupWeight = 0.4 * checkpointWeight;
    var miniWebBackupWeight = 0.2 * checkpointWeight;


    if (!forceNoBackup) {
        try {
            var cpuFwVersion = getCpuFwVersionStr();
            if (new VersionComparator().isNotLessThan(cpuFwVersion, minVersion)) {
                var backupFilePath = createBackup(converterScript, factoryDefault, restoreFromBackup, parentCheckpoint, configBackupWeight);
                enableHttp(); // GPI-181
                var presetFilesMetaDataList = createPresetBackup(skipPresetsAtRestore, converterScript, 10000, "bin", parentCheckpoint, presetBackupWeight);
                var indexFileMetaData;

                if (typeof uploadDefaultMiniWeb !== 'undefined') {
                    indexFileMetaData = createMiniWebBackup(10000, parentCheckpoint, miniWebBackupWeight);
                } else {
                    dummyCheckpointIncrease(parentCheckpoint, miniWebBackupWeight);
                }

                return {
                    backupFilePath: backupFilePath,
                    presetFilesMetaDataList: presetFilesMetaDataList,
                    indexFileMetaData: indexFileMetaData
                };
            } else {
                dummyCheckpointIncrease(parentCheckpoint, checkpointWeight);
                var msg = "Current FW on device doesn't support configuration backup creation.";
                logger.warn(msg);
                throw new UpdateFailedException(msg);
            }
        } catch (error) {
            if (isGui === true) {
                var userChoice = showBackupNotCreatedOptionDialog(deviceIp, uidEepromConfig, ownerWindow);
                dummyCheckpointIncrease(parentCheckpoint, checkpointWeight);
                if (userChoice === "CANCEL") {             // Cancel button pressed
                    throw new UpdateFailedException("Backup creation failed, user aborted upgrade.", error);
                } else {                            // OK button pressed
                    logger.warn(error);
                    logger.info("USER: Factory default is selected.");
                    factoryDefault = true;          // setting mutable global variable
                    restoreFromBackup = false;      // setting mutable global variable
                }
            } else {
                logger.warn("Backup could not be created.");
                throw error;
            }
        }
    } else {
        dummyCheckpointIncrease(parentCheckpoint, checkpointWeight);
    }
}

function showBackupNotCreatedOptionDialog(deviceIp, uidEepromConfig, ownerWindow) {
    var contentText = "Unable to make backup of device configuration.\n\n";
    contentText += "Click OK to continue the upgrade process and apply factory default settings. All user settings will be lost.\n\n";
    contentText += "Alternatively choose CANCEL to abort the upgrade process. No modification has been made to the device yet.";

    var devicePopupArgument = DevicePopupArgument.newBuilder()
        .setTcpConnection(new TcpConnection(deviceIp, 6107, 1000, 1000, 1))
        .setUidEepromConfig(uidEepromConfig)
        .setOwnerWindow(ownerWindow)
        .setButtons(["CANCEL", "OK"])
        .setContentText(contentText)
        .setHeaderText("Unable to make backup of device configuration")
        .setIconType(DevicePopupArgument.IconType.WARNING)
        .build();

    var devicePopupShow = new DevicePopupShow(devicePopupArgument);

    devicePopupShow.execute();

    return devicePopupShow.getClickedButton();
}

function enableHttp() {
    try {
        setPropertyLw3("HttpEnabled", "/MANAGEMENT/SERVICEFILTER", "true");
        setPropertyLw3("HttpPostEnabled", "/MANAGEMENT/SERVICEFILTER", "true");
        rootCheckpoint.waitMillisLong(1500);
        logger.info("HTTP and HTTP post have got enabled on port 80");
    } catch (e) {
        logger.debug("Failed to enable HTTP and/or HTTP post. The belonging properties may not exist.", e);
    }
}

