var createBackup = function (converterScriptName, factoryDefault, restoreFromBackup, parentCheckpoint, checkpointWeight) {
    var partNumber = '';
    var fwVersion = '';
    var fwRevision = '';

    var checkpoint = parentCheckpoint.add("Creating backup of device settings...", checkpointWeight);
    logger.info(checkpoint.getTitle());

    try {
        var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);

        var partNumberGetProperty;
        try {
            partNumberGetProperty = new GetProperty(lw3Connection, new Property("PartNumber", new Path("/MANAGEMENT/UID")));
            partNumberGetProperty.execute();
        } catch (e) {
            partNumberGetProperty = new GetProperty(lw3Connection, new Property("ProductPartNumber", new Path("/MANAGEMENT/UID")));
            partNumberGetProperty.execute();
        }
        partNumber = partNumberGetProperty.getProperty().getCachedValue();

        var firmwareVersionGetProperty;
        try {
            firmwareVersionGetProperty = new GetProperty(lw3Connection, new Property("Version", new Path("/MANAGEMENT/UID/PACKAGE/CPU")));
            firmwareVersionGetProperty.execute();
            fwVersion = firmwareVersionGetProperty.getProperty().getCachedValue();
        } catch (e) {
            firmwareVersionGetProperty = new GetProperty(lw3Connection, new Property("FirmwareVersion", new Path("/MANAGEMENT/UID")));
            firmwareVersionGetProperty.execute();
            fwVersion = firmwareVersionGetProperty.getProperty().getCachedValue().split(' ')[0];
        }

        try {
            var firmwareVcsGetProperty = new GetProperty(lw3Connection, new Property("Vcs", new Path("/MANAGEMENT/UID/PACKAGE/CPU")));
            firmwareVcsGetProperty.execute();
            fwRevision = firmwareVcsGetProperty.getProperty().getCachedValue();
        } catch (e) {
            var fwVersionWithRevision = firmwareVersionGetProperty.getProperty().getCachedValue().split(' ');
            if (fwVersionWithRevision.length === 1) {
                fwRevision = 0;
            } else {
                fwRevision = fwVersionWithRevision[1].substr(1);
            }
        }

        checkpoint.setProgress(0.2);

        lw3Connection.setReceive_timeout(10000);

        var backupCreateFlowArgumentBuilder = BackupCreateFlowArgument.newBuilder()
            .setTcpConnection(lw3Connection)
            .setBackupFolder(backupFolder)
            .addKeyValue("PARTNUMBER", partNumber)
            .addKeyValue("VERSION", fwVersion)
            .addKeyValue("REVISION", fwRevision);

        /* Set lw3 converter script */
        if (factoryDefault === false && restoreFromBackup === true) {
            backupCreateFlowArgumentBuilder.setConvertScriptInputStreamProvider(packageFile.getFileHandler(converterScriptName))
        }
        checkpoint.setProgress(0.4);

        var backupCreateFlowArgument = backupCreateFlowArgumentBuilder.build();
        var backupCreateFlow = new BackupCreateFlow(backupCreateFlowArgument);
        backupCreateFlow.setActionOnFailure(ActionOnFailure.ACTION_HALT);

        backupCreateFlow.execute();

        /* Save backup file path */
        var backupFilePath =  backupCreateFlow.getBackupFilePath();
        checkpoint.setProgress(1.0);
        return backupFilePath;

    } catch (exception) {
        var msg = "Failed to create backup of device settings.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }

}