var restoreSettings = function (lw3Connection, backupFilePath, applyNetworkSettings, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Restoring device settings from the following backup file: " + backupFilePath + " ...", checkpointWeight);
    logger.info(checkpoint.getTitle());
    try {
        if (backupFilePath === '') {
            throw new UpdateFailedException("Cannot restore device settings. Invalid backup file path");
        }

        var restoreNetworkSettingsArgument = RestoreNetworkSettingsArgument.newBuilder()
            .setRestoreNetworkMode(RestoreNetworkSettingsArgument.RestoreNetworkMode.BACKUP) // USE IP SETTINGS FROM BACKUP FILE
            .setApplyNetworkSettings(applyNetworkSettings)
            .build();

        /* Restore settings */
        var restoreOverLw3FlowArgument = RestoreOverLw3FlowArgument.newBuilder()
            .setTcpConnection(lw3Connection)
            .setBackupFilePath(backupFilePath)
            .setRestoreNetworkSettingsArgument(restoreNetworkSettingsArgument)
            .build();


        var restoreOverLw3Flow = new RestoreOverLw3Flow(restoreOverLw3FlowArgument);
        var progressIndicator = new ProgressIndicator();
        restoreOverLw3Flow.subscribeToProgress(
            function (progress) {
                checkpoint.setProgress(progress / 100.0);
                if (!isGui) {
                    progressIndicator.setProgress(parseInt(progress));
                }
            }
        );
        restoreOverLw3Flow.execute();

        logger.info("Device settings have been restored");
        checkpoint.setProgress(1.0);

    } catch (exception) {
        var msg = "Failed to restore device settings.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }
};
