var isIndexFileValid = function () {
    var indexFileValid = false;

    try {
        var lw3Connection = new TcpConnection(deviceIp, 6107, 2000, 2000, 3);

        var indexFileLengthGetProperty = new GetProperty(lw3Connection, new Property("ActualFileLen", new Path("/MANAGEMENT/INDEXFILE")));
        indexFileLengthGetProperty.execute();

        var length = parseInt(indexFileLengthGetProperty.getProperty().getCachedValue());
        indexFileValid = (0 < length);

    } catch (exception) {
        logger.info("Failed to get miniweb file length.");
    }

    return indexFileValid;
};


var createMiniWebBackup = function (httpSocketTimeout, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Creating miniweb backup...", checkpointWeight);
    try {
        var indexFileMetaDataList = [];
        if (isIndexFileValid()) {
            logger.info(checkpoint.getTitle());
            var indexFileMetaData = PresetFileMetaData.newBuilder()
                .setUrl("/index.html")
                .setName("index.html")
                .setBinaryContent(true)
                .build();
            indexFileMetaDataList.push(indexFileMetaData);
            logger.info("Making backup of the miniweb file.");
        } else {
            logger.info("No miniweb file is found on the device.");
        }

        var backupIndexFileFlowArgument = BackupPresetFlowArgument.newBuilder()
            .setDeviceIp(deviceIp)
            .setBackupFolderPath(backupFolder)
            .setPresetFileMetaDataList(indexFileMetaDataList)
            .setSocketTimeout(httpSocketTimeout)
            .build();

        var backupIndexFileFlow = new BackupPresetFlow(backupIndexFileFlowArgument);
        backupIndexFileFlow.execute();

        var presetFileMetaData = (indexFileMetaDataList.length === 0) ? null : backupIndexFileFlow.getPresetFileMetaDataList().get(0);
        checkpoint.setProgress(1.0);
        return presetFileMetaData;

    } catch (e) {
        var msg = "Failed to make backup of the miniweb file.";
        logger.warn(msg, e);
        throw new UpdateFailedException(msg, e);
    }
};