var uploadPackageSpecificMiniWeb = function (miniWebFileName, httpSocketTimeout, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Uploading package's default miniweb.", checkpointWeight);
    logger.info(checkpoint.getTitle());
    try {
        var httpProtocolConnection = new HttpProtocolConnection(HttpProtocolConnectionArguments
            .newBuilder()
            .setUri("http://" + deviceIp.getHostAddress() + "/index.html")
            .setSocketTimeout(httpSocketTimeout)
            .build());

        var httpArguments = HttpArguments.newBuilder()
            .setInputStreamProvider(packageFile.getFileHandler(miniWebFileName))
            .setPostName("index")
            .setUploadFileName("index.html")
            .build();

        var httpUpload = new HttpUploadFlow(httpProtocolConnection, httpArguments);
        httpUpload.execute();
        checkpoint.setProgress(1.0);

    } catch (exception) {
        var msg = "Failed to upload package specific miniweb.";
        logger.warn(msg, exception);
        throw new UpdateFailedException(msg, exception);
    }
};

var restoreCustomMiniWeb = function (indexFileMetaData, httpSocketTimeout, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Restoring custom miniweb file: " + indexFileMetaData.getName(), checkpointWeight);
    logger.info(checkpoint.getTitle());
    try {
        var restoreArgument = RestorePresetFlowArgument.newBuilder()
            .setDeviceIp(deviceIp)
            .setPresetFilePath(indexFileMetaData.getFilePath())
            .setRestoreUrl(indexFileMetaData.getUrl())
            .setSocketTimeout(httpSocketTimeout)
            .setRetries(3)
            .setPresetName(indexFileMetaData.getName())
            .build();

        var restoreFlow = new RestorePresetFlow(restoreArgument);
        restoreFlow.execute();
        checkpoint.setProgress(1.0);
    } catch (exception) {
        var msg = "Failed to restore custom miniweb.";
        logger.warn(msg, exception);
        throw new UpdateFailedException(msg, exception);
    }
};