include("common_scripts_lib/backup_and_restore/preset_commons.js");

var createPresetBackup = function (skipDevicePresets, converterScriptName, httpSocketTimeout, presetFileExtension, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Creating backup of device presets...", checkpointWeight);

    try {
        if (httpSocketTimeout < 1000) {
            throw new UpdateFailedException("createBackup() function is called with invalid argument: httpSocketTimeout.");
        }

        var partNumber = '';
        var fwVersion = '';
        var fwRevision = '';

        var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);

        var partNumberGetProperty;
        try {
            partNumberGetProperty = new GetProperty(lw3Connection, new Property("PartNumber", new Path("/MANAGEMENT/UID")));
            partNumberGetProperty.execute();
        } catch (e) {
            partNumberGetProperty = new GetProperty(lw3Connection, new Property("ProductPartNumber", new Path("/MANAGEMENT/UID")));
            partNumberGetProperty.execute();
        }
        partNumber = partNumberGetProperty.getProperty().getCachedValue();

        var firmwareVersionGetProperty;
        try {
            firmwareVersionGetProperty = new GetProperty(lw3Connection, new Property("Version", new Path("/MANAGEMENT/UID/PACKAGE/CPU")));
            firmwareVersionGetProperty.execute();
            fwVersion = firmwareVersionGetProperty.getProperty().getCachedValue();
        } catch (e) {
            firmwareVersionGetProperty = new GetProperty(lw3Connection, new Property("FirmwareVersion", new Path("/MANAGEMENT/UID")));
            firmwareVersionGetProperty.execute();
            fwVersion = firmwareVersionGetProperty.getProperty().getCachedValue().split(' ')[0];
        }

        try {
            var firmwareVcsGetProperty = new GetProperty(lw3Connection, new Property("Vcs", new Path("/MANAGEMENT/UID/PACKAGE/CPU")));
            firmwareVcsGetProperty.execute();
            fwRevision = firmwareVcsGetProperty.getProperty().getCachedValue();
        } catch (e) {
            var fwVersionWithRevision = firmwareVersionGetProperty.getProperty().getCachedValue().split(' ');
            if (fwVersionWithRevision.length === 1) {
                fwRevision = 0;
            } else {
                fwRevision = fwVersionWithRevision[1].substr(1);
            }
        }

        checkpoint.setProgress(0.2);

        var presetFileMetaDataList = [];

        for (presetId = 1; presetId <= getPresetCount(); presetId++) {

            var presetProperties = getPresetProperties(presetId);
            logger.debug("Preset {} properties: {}", presetId, JSON.stringify(presetProperties));

            if (false === presetProperties['empty']) {
                var presetFileMetaDataBuilder = PresetFileMetaData.newBuilder()
                    .setUrl("/preset" + presetId + "." + presetFileExtension)
                    .setName(presetProperties['name'] + " (" + "preset" + presetId + ")");

                if (true === presetProperties['convertable']) {
                    if (!factoryDefault && !skipDevicePresets) {
                        presetFileMetaDataBuilder.setConvertScriptInputStreamProvider(packageFile.getFileHandler(converterScriptName))
                    }
                } else {
                    /* Base64 encode content */
                    presetFileMetaDataBuilder.setBinaryContent(true);
                }

                var presetFileMetaData = presetFileMetaDataBuilder.build();
                presetFileMetaDataList.push(presetFileMetaData);

                logger.info("Backup of device preset {} is queued", presetFileMetaData.getName());
            }

        }

        var backupPresetFlowArgument = BackupPresetFlowArgument.newBuilder()
            .setDeviceIp(deviceIp)
            .setBackupFolderPath(backupFolder)
            .setPresetFileMetaDataList(presetFileMetaDataList)
            .addKeyValue("PARTNUMBER", partNumber)
            .addKeyValue("VERSION", fwVersion)
            .addKeyValue("REVISION", fwRevision)
            .setSocketTimeout(httpSocketTimeout)
            .build();

        if (presetFileMetaDataList.length > 0) {
            logger.info(checkpoint.getTitle());
        }

        var backupPresetFlow = new BackupPresetFlow(backupPresetFlowArgument);
        backupPresetFlow.execute();

        var presetFileMetaDataList = backupPresetFlow.getPresetFileMetaDataList();
        checkpoint.setProgress(1.0);
        return presetFileMetaDataList

    } catch (exception) {
        var msg = "Failed to create backup of device configuration preset.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }
};