include("common_scripts_lib/backup_and_restore/preset_commons.js");

var deletePreset = function (presetId, socketTimeout /* ms */) {
    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, socketTimeout, 3);
    if (socketTimeout < 1000) {
        throw new UpdateFailedException("deletePreset() function is called with invalid argument: socketTimeout");
    }

    logger.info("Deleting preset: preset{}", presetId);
    var deletePresetMethod = new Method("delete", new Path("/PRESETS/DEVICE"));
    var callDeletePreset = new CallMethod(lw3Connection, deletePresetMethod, presetId + ";IDDQD");
    callDeletePreset.execute();

    var presetFileLengthGetProperty = new GetProperty(lw3Connection, new Property("fileLen", new Path("/PRESETS/DEVICE/" + presetId)));
    presetFileLengthGetProperty.execute();
    var fileLength = parseInt(presetFileLengthGetProperty.getProperty().getCachedValue());

    if (0 !== fileLength) {
        throw new UpdateFailedException("Failed to delete preset " + presetId);
    }
};

var deleteWriteProtectedPresets = function (presetEraseTimeout, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Erasing write protected presets...", checkpointWeight);
    try {
        if (presetEraseTimeout < 1000) {
            throw new UpdateFailedException("deleteWriteProtectedPresets() function is called with invalid argument: presetEraseTimeout");
        }

        var erasePresetList = [];

        for (var presetId = 1; presetId <= getPresetCount(); presetId++) {

            var presetProperties = getPresetProperties(presetId);
            if (true === presetProperties['writeProtected']) {
                erasePresetList.push(presetId);
            }
        }
        if (erasePresetList.length > 0) {
            logger.info(checkpoint.getTitle());
        }
        erasePresetList.forEach(
            function (presetId) {
                logger.debug("Deleting preset {}", presetId);
                deletePreset(presetId, presetEraseTimeout);
            });

        checkpoint.setProgress(1.0);

    } catch (exception) {
        var msg = "Failed to erase protected device preset.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }
};

var deleteNonEmptyPresets = function (presetEraseTimeout, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Erasing non-empty presets...", checkpointWeight);
    try {
        if (presetEraseTimeout < 1000) {
            throw new UpdateFailedException("deleteWriteProtectedPresets() function is called with invalid argument: presetEraseTimeout");
        }

        var erasePresetList = [];

        for (var presetId = 1; presetId <= getPresetCount(); presetId++) {

            var presetProperties = getPresetProperties(presetId);
            if (false === presetProperties['empty']) {
                erasePresetList.push(presetId);
            }
        }
        if (erasePresetList.length > 0) {
            logger.info(checkpoint.getTitle());
        }
        erasePresetList.forEach(
            function (presetId) {
                logger.debug("Deleting preset {}", presetId);
                deletePreset(presetId, presetEraseTimeout);
            });

        checkpoint.setProgress(1.0);

    } catch (exception) {
        var msg = "Failed to erase device preset.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }
};
