include("common_scripts_lib/backup_and_restore/preset_delete.js");

var restorePresets = function (fileMetaDataList, httpSocketTimeout /* ms */, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Restoring device presets...", checkpointWeight);
    try {
        if (!(fileMetaDataList instanceof Array) && !(fileMetaDataList instanceof java.util.ArrayList)) {
            throw new UpdateFailedException("restorePresets() function is called with invalid argument: fileMetaDataList");
        }
        if (httpSocketTimeout < 1000) {
            throw new UpdateFailedException("restorePresets() function is called with invalid argument: httpSocketTimeout");
        }

        if (fileMetaDataList.length > 0) {
            logger.info(checkpoint.getTitle());
        }
        fileMetaDataList.forEach(
            function (fileMetaData) {
                var restoreTrials = 0;
                var MAX_TRIALS = 10;

                while (restoreTrials < MAX_TRIALS) {
                    restoreTrials++;
                    try {
                        deletePresetByFileMetaData(fileMetaData);
                        restorePreset(fileMetaData, httpSocketTimeout);
                        var presetValid = validatePreset(fileMetaData);
                        logger.debug("All uploaded presets are valid: {}", presetValid);
                        if (presetValid) {
                            checkpoint.setProgress(checkpoint.getProgress() + (1.0 / fileMetaDataList.length));
                            break;
                        }
                        if (restoreTrials === MAX_TRIALS) {
                            logger.warn("Uploaded preset is not valid: {}", fileMetaData.getUrl());
                            break;
                        } else {
                            logger.warn("Preset upload failed ({}) at validation. Retrying.", fileMetaData.getUrl());
                        }
                    } catch (exception) {
                        if (restoreTrials === MAX_TRIALS) {
                            logger.warn("Preset upload failed after multiple trials.", exception);
                            throw exception;
                        } else {
                            logger.warn("Preset upload failed with exception. Retrying.", exception);
                        }
                    }
                }
            });

        checkpoint.setProgress(1.0);

    } catch (exception) {
        var msg = "Failed to restore device presets.";
        logger.warn(msg);
        throw new UpdateFailedException(msg, exception);
    }
};

function restorePreset(fileMetaData, httpSocketTimeout) {
    var restoreArgument = RestorePresetFlowArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setPresetFilePath(fileMetaData.getFilePath())
        .setRestoreUrl(fileMetaData.getUrl())
        .setSocketTimeout(httpSocketTimeout)
        .setRetries(3)
        .setPresetName(fileMetaData.getName())
        .build();

    logger.info("Restoring preset: {}", fileMetaData.getName());

    var restoreFlow = new RestorePresetFlow(restoreArgument);
    restoreFlow.execute();
    rootCheckpoint.waitMillisLong(1000);
}

function deletePresetByFileMetaData(fileMetaData) {
    var presetId = fileMetaData.getUrl().split("/preset")[1].split(".")[0];
    deletePreset(presetId, 90000);
}

function validatePreset(fileMetaData) {
    var os = new ByteArrayOutputStream();
    var downloadArguments = HttpArguments.newBuilder()
        .setOutputStream(os)
        .build();

    var connection = new HttpProtocolConnection(
        HttpProtocolConnectionArguments.newBuilder()
            .setRetries(3)
            .setUri("http://" + deviceIp.getHostAddress() + fileMetaData.getUrl())
            .setSocketTimeout(10000)
            .build());

    var download = new HttpDownload(connection, downloadArguments);
    download.execute();

    var downloadedPreset = removeCommentAndEmptyLines(os.toString());

    var File = Java.type("java.io.File");
    var StandardCharsets = Java.type("java.nio.charset.StandardCharsets");
    var FileUtils = Java.type("org.apache.commons.io.FileUtils");

    var file = new File(fileMetaData.getFilePath());
    var fileContent = removeCommentAndEmptyLines(FileUtils.readFileToString(file, StandardCharsets.UTF_8));

    var validPreset = fileContent.equals(downloadedPreset);
    logger.debug("Uploaded preset ({}, {}) is valid: {}", fileMetaData.getUrl(), fileMetaData.getName(), validPreset);
    return validPreset;
}

function removeCommentAndEmptyLines(text) {
    var StringBuilder = Java.type("java.lang.StringBuilder");
    var validRestoreSb = new StringBuilder();
    var lines = text.split("\n");
    for (var i = 0; i < lines.length; i++) {
        var line = lines[i];
        line = line.trim();
        if (!(line.equals("") || line.startsWith(";"))) {
            validRestoreSb.append(line);
            validRestoreSb.append("\r\n");
        }
    }
    return validRestoreSb.toString();
}