include("common_scripts_lib/legacy/package.js");
include("common_scripts_lib/legacy/uid.js");
include("common_scripts_lib/ldu2p/control.js");

var DOWNGRADE_USER_CANCELLED_MSG = "The firmware on the device is newer than the one in the package. It is not possible to restore settings. User cancelled update.";
var DOWNGRADE_FACTORY_MSG = "The firmware package on the device is newer than the selected package. It is not possible to restore settings, all settings will be set to factory default values.";

function showDowngradeOptionDialog(deviceIp, uidEepromConfig, ownerWindow) {
    var contentText = "The firmware on the device is newer than the one in the package. It is not possible to restore device configuration. All settings will be set to factory default values.\n\n";
    contentText += "Click OK to go on with factory settings, or click CANCEL to abort the upgrade process. (No modification has been made to the device yet.)";

    var devicePopupArgument = DevicePopupArgument.newBuilder()
        .setTcpConnection(new TcpConnection(deviceIp, 6107, 1000, 1000, 1))
        .setUidEepromConfig(uidEepromConfig)
        .setOwnerWindow(ownerWindow)
        .setButtons(["CANCEL", "OK"])
        .setContentText(contentText)
        .setHeaderText("Newer firmware detected on device")
        .setIconType(DevicePopupArgument.IconType.WARNING)
        .build();

    var devicePopupShow = new DevicePopupShow(devicePopupArgument);

    devicePopupShow.execute();

    return devicePopupShow.getClickedButton();
}

function checkDowngradeLDU1(deviceIp, applicationFile, uidEepromConfig, ownerWindow) {
    /* Check if the current firmware on the device is newer then the one in the package */
    if (!factoryDefault && (isCurrentFirmwareNewer(applicationFile))) {
        if (isGui) {
            var userChoice = showDowngradeOptionDialog(deviceIp, uidEepromConfig, ownerWindow);
            if (userChoice === "CANCEL") {
                var bootCtrl = new BootCtrl(deviceIp);
                bootCtrl.runApplication();
                logger.warn(DOWNGRADE_USER_CANCELLED_MSG);
                throw new UpdateFailedException("User cancelled update.");
            } else if (userChoice === "OK") {
                logger.warn(DOWNGRADE_FACTORY_MSG);
                factoryDefault = true;
            }
        } else {
            logger.warn(DOWNGRADE_FACTORY_MSG);
            factoryDefault = true;
        }
    }
}

function checkDowngradeLDU2(connection, packageVersionString, packageVersionOnDeviceString){
    if (!factoryDefault && !(new VersionComparator().isNotLessThan(packageVersionString, packageVersionOnDeviceString))) {
        if (isGui) {
            var userChioce = showDowngradeOptionDialog(deviceIp, null, ownerWindow);
            if (userChioce === "CANCEL") {
                enterApplication(connection);
                logger.warn(DOWNGRADE_USER_CANCELLED_MSG);
                throw new UpdateFailedException("User cancelled update.");
            } else if (userChioce === "OK") {
                logger.warn(DOWNGRADE_FACTORY_MSG);
                factoryDefault = true;
            }
        } else {
            logger.warn(DOWNGRADE_FACTORY_MSG);
            factoryDefault = true;
        }
    }
}

function checkDowngradeGeneral(packageVersionString, packageVersionOnDeviceString){
    if (!factoryDefault && !(new VersionComparator().isNotLessThan(packageVersionString, packageVersionOnDeviceString))) {
        if (isGui) {
            var userChioce = showDowngradeOptionDialog(deviceIp, null, ownerWindow);
            if (userChioce === "CANCEL") {
                logger.warn(DOWNGRADE_USER_CANCELLED_MSG);
                throw new UpdateFailedException("User cancelled update.");
            } else if (userChioce === "OK") {
                logger.warn(DOWNGRADE_FACTORY_MSG);
                factoryDefault = true;
            }
        } else {
            logger.warn(DOWNGRADE_FACTORY_MSG);
            factoryDefault = true;
        }
    }
}
