include("common_scripts_lib/lw3/control_lw3.js");

function isLoginSupported() {
    try {
        getPropertyLw3("LoginEnable", "/LOGIN");
        return true;
    } catch (e) {
        return false;
    }
}

function isLoginEnabled() {
    try {
        return getPropertyLw3("LoginEnable", "/LOGIN") === "true";
    } catch (e) {
        return false;
    }
}

function isLoggedIn() {
    return getPropertyLw3("LoggedIn", "/LOGIN").toLowerCase() === "true";
}

function loginAndUnclock(clearTextPw) {
    var connection = new TcpConnection(deviceIp, 6107);

    var loginMethod = new Method("login", new Path("/LOGIN"));
    loginMethod.setCachedOutput(clearTextPw);

    var loginEnableProperty = new Property("LoginEnable", new Path("/LOGIN"));
    loginEnableProperty.setCachedValue("false");

    var ArrayList = Java.type('java.util.ArrayList');
    var executionList = new ArrayList();
    executionList.add(loginMethod);
    executionList.add(loginEnableProperty);

    var loginAndUnlock = new BatchRestore(connection, executionList);

    loginAndUnlock.execute();

    if (!loginAndUnlock.getExceptionResponses().isEmpty()) {
        if (!isLoggedIn()) {
            var exception = new UpdateFailedException("Failed to log in.");
            exception.addContextValue("Error responses received during login", loginAndUnlock.getExceptionResponses());
            throw exception;
        } else {
            logger.info("Already logged in. {}", loginAndUnlock.getExceptionResponses());
        }
    }
}

function setLoginEnable() {
    setPropertyLw3("LoginEnable", "/LOGIN", "true");
}

function showPwInputPopup() {
    var devicePopupArgument = DevicePopupArgument.newBuilder()
        .setTcpConnection(new TcpConnection(deviceIp, 6107, 1000, 1000, 1))
        .setUidEepromConfig(UID_FVS_EEPROM_CONFIG)
        .setOwnerWindow(ownerWindow)
        .setButtons(["OK", "CANCEL"])
        .setContentText("Enter password to log in.")
        .setHeaderText("Login needed")
        .setTextInput("Password:")
        .setIconType(DevicePopupArgument.IconType.QUESTION)
        .build();

    var devicePopupShow = new DevicePopupShow(devicePopupArgument);

    devicePopupShow.execute();

    if (devicePopupShow.getClickedButton() === "CANCEL") {
        throw new UpdateFailedException("Update cancelled.");
    } else {
        return devicePopupShow.getTextInputContent();
    }
}

function showInvalidPwPopup() {
    var devicePopupArgument = DevicePopupArgument.newBuilder()
        .setTcpConnection(new TcpConnection(deviceIp, 6107, 1000, 1000, 1))
        .setUidEepromConfig(UID_FVS_EEPROM_CONFIG)
        .setOwnerWindow(ownerWindow)
        .setButtons(["OK"])
        .setContentText("The given login password is invalid.")
        .setHeaderText("Invalid login password")
        .setIconType(DevicePopupArgument.IconType.ERROR)
        .build();

    var devicePopupShow = new DevicePopupShow(devicePopupArgument);

    devicePopupShow.execute();
}

function logIn(cleartextLoginPw) {
    if (!isLw3Capable()) return false;

    logLoginStatus();

    var loginSupported = isLoginSupported();
    var loginEnabled = isLoginEnabled();

    if (loginEnabled && !isLoggedIn()) {
        if (cleartextLoginPw !== "") {
            try {
                loginAndUnclock(cleartextLoginPw);
                logger.info("Logged in successfully.")
            } catch (exception) {
                if (isGui) {
                    showInvalidPwPopup();
                    cleartextLoginPw = showPwInputPopup();
                    logIn(cleartextLoginPw);
                } else {
                    throw exception;
                }
            }
        } else {
            if (isGui) {
                cleartextLoginPw = showPwInputPopup();
                logIn(cleartextLoginPw);
            } else {
                throw new UpdateFailedException("Cleartext login is enabled on the device. Please use \"--scriptparam clearTextLoginPw=yourPasswordHere\" command line switch.");
            }
        }
    } else if (loginSupported && !loginEnabled) {
        logger.info("Login functionality is currently not enabled.");
    } else if (loginEnabled && isLoggedIn()) {
        logger.warn("Already logged in."); // invalid state
    } else {
        logger.debug("Login functionality is not supported.");
    }
    return loginEnabled;
}

function logLoginStatus() {
    var deviceLoginStatus = "Login status: ";
    deviceLoginStatus += "isLoginSupported: " + isLoginEnabled();
    deviceLoginStatus += "isLoginEnabled: " + isLoginEnabled();
    if (isLoginSupported()) {
        deviceLoginStatus += "isLoggedIn: " + isLoggedIn();
    }
    logger.debug(deviceLoginStatus);
}





