function logFirmwareVersion(connection, descriptor) {
    var getFirmwareVersion = new GetFirmwareVersion(connection, descriptor.fileNameAtDevice);
    getFirmwareVersion.execute();

    var firmwareVersionArgumentFromDevice = getFirmwareVersion.getFirmwareVersionArgument();
    var firmwareVersionArgumentInPackage = packageFile.getFirmwareVersionArgument(descriptor.fileNameInPackage);

    logger.info("Firmware: {} ({})\n\tActual version: \t{}\n\tNew version: \t\t{}", descriptor.title, descriptor.fileNameAtDevice,
        ArgumentFormatter.formatFirmwareVersion(firmwareVersionArgumentFromDevice),
        ArgumentFormatter.formatFirmwareVersion(firmwareVersionArgumentInPackage));
}

function compareFirmwareVersion(connection, descriptor) {
    var getFirmwareVersion = new GetFirmwareVersion(connection, descriptor.fileNameAtDevice);
    getFirmwareVersion.execute();

    var firmwareVersionArgumentFromDevice = getFirmwareVersion.getFirmwareVersionArgument();
    var firmwareVersionArgumentInPackage = packageFile.getFirmwareVersionArgument(descriptor.fileNameInPackage);

    logger.info("Firmware: {} ({})\n\tActual version: \t{}\n\tNew version: \t\t{}", descriptor.title, descriptor.fileNameAtDevice,
        ArgumentFormatter.formatFirmwareVersion(firmwareVersionArgumentFromDevice),
        ArgumentFormatter.formatFirmwareVersion(firmwareVersionArgumentInPackage));

    return firmwareVersionArgumentFromDevice.equals(firmwareVersionArgumentInPackage);
}

function setFirmwareVersion(connection, descriptor) {
    var firmwareVersionArgumentInPackage = packageFile.getFirmwareVersionArgument(descriptor.fileNameInPackage);

    logger.info("Setting firmware version on device for file: {} to {}", descriptor.fileNameAtDevice, ArgumentFormatter.formatFirmwareVersion(firmwareVersionArgumentInPackage));

    var setFirmwareVersionFlow = new SetFirmwareVersionFlow(connection, packageFile.getFirmwareVersionArgument(descriptor.fileNameInPackage), descriptor.fileNameAtDevice);
    setFirmwareVersionFlow.execute();
}

function enterBootload(connection) {
    logger.info("Switching device to Firmware Update mode...");

    rootCheckpoint.indeterminate();

    try {
        var lw3Connection = new TcpConnection(deviceIp, 6107);
        var bootloadPath = new Path("/SYS");
        var bootloadMethod = new Method("bootload", bootloadPath);
        var bootloadMethodCall = new CallNoReturnMethod(lw3Connection, bootloadMethod, "");
        bootloadMethodCall.execute();
        rootCheckpoint.waitMillisLong(1000);

    } catch (e) {
        logger.warn("Failed to execute LW3 command, the device might be already in Update mode.");
    }

    var _connectionParameters = new Ldu2ProtocolConnectionParameters(
        connection.getDestination().getIpAddressAndPort(),
        25,
        1);

    var _connection = new Ldu2ProtocolConnection(_connectionParameters);

    var getMode = new GetMode(_connection);
    getMode.execute();

    if (getMode.getMode() !== DeviceMode.DEVICE_UPDATE_MODE) {
        logger.info("Switching to update mode...");

        var modeArgument = ModeArgument.newBuilder()
            .setDeviceMode(DeviceMode.DEVICE_UPDATE_MODE)
            .setMinBootTime(5)
            .setMaxBootTime(30)
            .build();
        var setModeFlow = new SetModeFlow(connection, modeArgument);
        setModeFlow.execute();
    }
}

function enterApplication(connection) {
    logger.info("Switching device to application mode...");

    var deviceMode;
    if (factoryDefault) {
        deviceMode = DeviceMode.DEVICE_APPLICATION_FACTORY_DEFAULT_MODE;
    } else {
        deviceMode = DeviceMode.DEVICE_APPLICATION_MODE;
    }
    var modeArgument = ModeArgument.newBuilder()
        .setDeviceMode(deviceMode)
        .setMinBootTime(5)
        .setMaxBootTime(15)
        .build();
    var setModeFlow = new SetModeFlow(connection, modeArgument);
    setModeFlow.execute();

}

function getPackageVersionOnDevice(connection){
    logger.info("Getting currently installed package's version...");
    var getPackageVersion = new GetPackageVersion(connection);
    getPackageVersion.execute();
    var packageVersionArgument = getPackageVersion.getPackageVersionArgument();
    return ArgumentFormatter.formatPackageVersion(packageVersionArgument);
}