function uploadAndVerifyFirmware (packageFile, connection, descriptor, parentCheckpoint, checkpointWeight) {
    if (descriptor.eraseBeforeUpload) {
        var erase_checkpoint = parentCheckpoint.add("Erasing " + descriptor.fileNameAtDevice, checkpointWeight * 0.4);
        var upload_checkpoint = parentCheckpoint.add("Uploading and verifying " + descriptor.fileNameAtDevice, checkpointWeight* 0.6);

        logger.info("Erasing {}", descriptor.title);

        var eraseFirmwareFlow = new EraseFirmwareFlow(connection, descriptor.fileNameAtDevice, 0);
        var progressIndicator = new ProgressIndicator();

        eraseFirmwareFlow.subscribeToEraseStatus(
            function (eraseStatus) {
                var erased = eraseStatus.getErasedBlockCount();
                var total = eraseStatus.getTotalBlockCount();
                progressIndicator.setProgress(parseInt((100 * erased) / total));

                erase_checkpoint.setProgress(erased / total);
            }
        );

        eraseFirmwareFlow.execute();

        erase_checkpoint.setProgress(1.0);
    }
    else {
        var upload_checkpoint = parentCheckpoint.add("Uploading and verifying " + descriptor.fileNameAtDevice, checkpointWeight * 1.0);
    }

    logger.info("Uploading and verifying {} ({})", descriptor.title, descriptor.fileNameInPackage)

    var tftpPutFlow = new TftpPutFlow(TftpInputStreamArgument.newBuilder()
        .setHostIp(connection.getDestination().getIpAddressAndPort().getAddress().getHostAddress())
        .setFileName(descriptor.fileNameAtDevice)
        .setTftpMode(1)
        .setInputStreamProvider(packageFile.getFileHandler(descriptor.fileNameInPackage))
        .build());
    var progressIndicator = new ProgressIndicator();

    tftpPutFlow.subscribeToProgress(
        function (progress) {
            progressIndicator.setProgress(parseInt(100 * (progress.uploadedBytes + progress.downloadedBytes) / (progress.totalBytes * 2)));
            upload_checkpoint.setProgress((progress.uploadedBytes + progress.downloadedBytes) / (progress.totalBytes * 2));
        }
    );

    tftpPutFlow.execute();
    upload_checkpoint.setProgress(1.0);
}

var updateFirmwareUpdater = function (connection, fwuFileSet, parentCheckpoint, checkpointWeight) {
    logger.info("Updating FirmwareUpdater");

    var modeArgument = ModeArgument.newBuilder()
        .setDeviceMode(DeviceMode.DEVICE_PREPARE_FIRMWARE_UPDATER_UPDATE_MODE)
        .setMinBootTime(8)
        .setMaxBootTime(30)
        .build();
    var setModeFlow = new SetModeFlow(connection, modeArgument);
    setModeFlow.execute();

    for (var i = 0; i < fwuFileSet.length; i++) {
        var descriptor = {
            title: fwuFileSet[i].descriptor.title,
            fileNameAtDevice: fwuFileSet[i].stageFile,
            fileNameInPackage: fwuFileSet[i].descriptor.fileNameInPackage,
            eraseBeforeUpload: fwuFileSet[i].eraseBeforeUpload
        };

        logger.debug(descriptor);

        var fileUpdate_checkpoint = parentCheckpoint.add("Updating " + descriptor.title, checkpointWeight / fwuFileSet.length);

        logFirmwareVersion(connection, descriptor);
        uploadAndVerifyFirmware(packageFile, connection, descriptor, fileUpdate_checkpoint, 1.0);
        setFirmwareVersion(connection, descriptor);
    }

    logger.info("Restarting device");

    var setMode = new SetMode(connection, DeviceMode.DEVICE_FIRMWARE_UPDATER_UPDATE_MODE);
    setMode.execute();

    rootCheckpoint.indeterminate();
    rootCheckpoint.waitMillisLong(3000);

    var _connectionParameters = new Ldu2ProtocolConnectionParameters(
        connection.getDestination().getIpAddressAndPort(),
        30,
        1);

    var _connection = new Ldu2ProtocolConnection(_connectionParameters);

    var getMode = new GetMode(_connection);
    getMode.execute();

    if (getMode.getMode() !== DeviceMode.DEVICE_UPDATE_MODE) {
        throw new UpdateFailedException("Updating FirmwareUpdater failed, device is not responding.");
    }
}

function updateFirmwareUpdaterIfNecessary(connection, fwuFileSet, parentCheckpoint, checkpointWeight) {

    var fwu_checkpoint = parentCheckpoint.add("Updating FirmwareUpdater", checkpointWeight);

    if (forceFWUUpdate) {
        updateFirmwareUpdater(connection, fwuFileSet, fwu_checkpoint, 1.0);
    } else {
        logger.info("Checking FirmwareUpdater");
        var updateFWUNeeded = false;

        for (var i = 0; i < fwuFileSet.length; i++) {
            updateFWUNeeded |= !compareFirmwareVersion(connection, fwuFileSet[i].descriptor);
        }
        if (updateFWUNeeded) {
            updateFirmwareUpdater(connection, fwuFileSet, fwu_checkpoint, 1.0);
        } else {
            logger.info("FirmwareUpdater is up-to-date");
            fwu_checkpoint.setProgress(1.0);
        }
    }
}

function updateFirmwareFiles(connection, packageFile, updateFileSet, parentCheckpoint, checkpointWeight) {
    var update_checkpoint = parentCheckpoint.add("Updating firmwares", checkpointWeight);

    for (var i = 0; i < updateFileSet.length; i++) {
        var descriptor = updateFileSet[i];
        logger.debug(descriptor);

        var fileUpdate_checkpoint = update_checkpoint.add("Updating " + descriptor.title, 1.0 / updateFileSet.length);

        logFirmwareVersion(connection, descriptor);
        uploadAndVerifyFirmware(packageFile, connection, descriptor, fileUpdate_checkpoint, 1.0);
        setFirmwareVersion(connection, descriptor);
    }
}


