function resetDevice(deviceIp, enableReset, resetLw3Method, minBootTime, maxBootTime, parentCheckpoint, resetWeight) {
    var checkpoint = parentCheckpoint.add("Resetting device", resetWeight);
    if (enableReset) {
        logger.info("Resetting device...");
        var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
        new CallNoReturnMethod(connection, resetLw3Method, "").execute();
        rootCheckpoint.waitMillisLong(1000);
        waitForDeviceToStartApplication(minBootTime, maxBootTime);
        checkpoint.setProgress(1.0);
    } else {
        checkpoint.setProgress(1.0);
    }
}

var waitForDeviceToStartApplication = function (minBootTime /* ms */, maxBootTime /* ms */) {
    rootCheckpoint.indeterminate();
    if ((minBootTime < 500)) {
        throw "waitForDeviceToStartApplication() function is called with invalid argument: minBootTime."
    }
    if ((maxBootTime < 1500) || (maxBootTime < minBootTime)) {
        throw "waitForDeviceToStartApplication() function is called with invalid argument: maxBootTime."
    }

    rootCheckpoint.waitMillisLong(minBootTime);

    var retries = Math.ceil((maxBootTime - minBootTime) / 1000);

    for (var i = 0; i < retries; i++) {
        try {
            var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
            new GetNodeNodes(connection, new Node(new Path("/"))).execute();
            return;

        } catch (exception) {
            logger.trace("Device is not available over LW3 protocol yet. Retrying...({})", i);
        }

        rootCheckpoint.waitMillisLong(1000);
    }
    throw new UpdateFailedException("Could not connect to device after starting application.");
};

var enterBootload = function () {
    logger.info("Switching device into bootload mode...");
    try {
        var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
        var bootloadMethod = new Method("bootload", new Path("/SYS"));

        new CallNoReturnMethod(connection, bootloadMethod, "").execute();

        rootCheckpoint.waitMillisLong(1000);

    } catch (e) {
        new BootCtrl(deviceIp).resetBootloader();
    }
};

function isLw3Capable(){
    try {
        var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
        new GetNodeNodes(connection, new Node(new Path("/"))).execute();
        return true;
    } catch (e) {
        return false;
    }
}

function getPropertyLw3(propertyName, propertyPath) {
    var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
    var getProperty = new GetProperty(connection, new Property(propertyName, new Path(propertyPath)));
    getProperty.execute();
    return getProperty.getProperty().getCachedValue();
}

function setPropertyLw3(propertyName, propertyPath, value) {
    var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 1);
    var setProperty = new SetProperty(connection, new Property(propertyName, new Path(propertyPath)), value);
    setProperty.execute();
}