include("common_scripts_lib/backup_and_restore/backup_all.js");
include("common_scripts_lib/backup_and_restore/restore_all.js");

include("common_scripts_lib/check/checkDowngrade.js");

include("common_scripts_lib/legacy/control.js");
include("common_scripts_lib/lw3/control_lw3.js");
include("common_scripts_lib/legacy/tftp.js");
include("common_scripts_lib/legacy/package.js");
include("common_scripts_lib/legacy/uid.js");

include("common_scripts_lib/cleartext_login/cleartextLogin.js");

/* Device specific declarations */
include("mmx4x2_hw.js");

/* TPSRX200-2841 - replace UPB Extension if needed */
include("TPSRX200_2841_helpers.js");

var PN_MMX4x2_HDMI = "91310034";
var PN_MMX4x2_HT200 = "91310035";
var PN_HDMI_TPS_RX200_VER_ENG = "91540039";

/* Create a logger */
var logger = LoggerFactory.getLogger("MMX4x2 update script");


/* main function */
var main = function () {
    rootCheckpoint.indeterminate();

    /* print script parameters */
    printScriptParams();

    /* print package version */
    logger.info("Package version: " + packageFile.getPackageVersionArgument().getMajorVersion() + "." + packageFile.getPackageVersionArgument().getMinorVersion() + "." + packageFile.getPackageVersionArgument().getHotfixNumber() + "b" + packageFile.getPackageVersionArgument().getRcNumber());

    var loginEnabled = logIn(clearTextLoginPw);

    /* make backups */
    var backups = makeFullConfigurationBackup("1.1.0b3", "tps_rx200.js", rootCheckpoint, 0.05, deviceIp, UID_FVS_EEPROM_CONFIG, ownerWindow);
    if (backups) {
        var backupFilePath = backups.backupFilePath;
        var presetFilesMetaDataList = backups.presetFilesMetaDataList;
        var indexFileMetaData = backups.indexFileMetaData;
    }

    enableHttp(); // GPI-181

    var partNumber = getPartNumber();


    /* enter bootload mode */
    try {
        try {
            var partNumber = getPartNumber();
            var packageVersion = getPackageVersionLW3AndUpb();
        } catch (ee) {
            showRestartYourDevicePopup(deviceIp, UID_FVS_EEPROM_CONFIG, ownerWindow);
            throw new UpdateFailedException("Unable to communicate with the device. Try switching the device into application mode. And start the update process again.", ee);
        }

        if (partNumber === PN_MMX4x2_HDMI && new VersionComparator().isInRange(packageVersion, "1.3.1b6", "1.3.4b3")) { // TPSRX200-2841
            if (isLw3Capable()) {
                enterBootloadNoExtension(); // TPSRX200-2841
                if (!isUpbExtensionUpToDate(UPB_EXTENSION)) { // TPSRX200-2841
                    replaceUpbExtension(UPB_EXTENSION, rootCheckpoint, 0.0); // TPSRX200-2841
                }
            } else {
                if (!isUpbExtensionUpToDate(UPB_EXTENSION)) { // TPSRX200-2841
                    showRestartYourDevicePopup(deviceIp, UID_FVS_EEPROM_CONFIG, ownerWindow);
                    throw new UpdateFailedException("UPB extension needs to be replaced. Switch the device into application mode. And start the update process again.");
                }
            }

        } else if (checkCpuFwVersion("v1.3.4b1")) {
            enterBootload();
        } else {
            enterBootloadHttp(); // UTT-1374
        }
    } catch (e) {
        if (e instanceof UpdateFailedException) {
            throw e;
        } else {
            new BootCtrl(deviceIp).resetBootloader();
        }
    }

    rootCheckpoint.waitMillisLong(12000); // UTT-1428, LDU2-1676

    /* get UID information from device */
    var uid = getUID(UID_FVS_EEPROM_CONFIG, rootCheckpoint, 0.05);
    printUid(uid);

    logger.info("Package version on device: {}", getPackageVersion(UID_FVS_EEPROM_CONFIG));

    partNumber = uid.getProductPartNumber();

    /* check whether about to downgrade */
    checkDowngradeLDU1(deviceIp, APPLICATION_FILE, UID_FVS_EEPROM_CONFIG, ownerWindow);


    /* upload files */
    uploadFiles(partNumber, rootCheckpoint, 0.50);


    /* Set package version */
    setPackageVersion(UID_FVS_EEPROM_CONFIG, rootCheckpoint, 0.05);

    /* Start application */
    startApplication(UID_FVS_EEPROM_CONFIG, backupFilePath && restoreFromBackup && !factoryDefault, 1000, 90000, 60, rootCheckpoint, 0.2);

    /* Restore configuration */
    restoreFullConfiguration(deviceIp, factoryDefault, partNumber, restoreFromBackup, skipPresetsAtRestore, backupFilePath, presetFilesMetaDataList, indexFileMetaData, false, loginEnabled, rootCheckpoint, 0.04, 0.04, 0.02);

    /* Reset device */
    resetDevice(deviceIp, !factoryDefault, new Method("reset", new Path("/SYS")), 2000, 30000, rootCheckpoint, 0.05);

    rootCheckpoint.finish();
};

function printScriptParams() {
    var params = "Script parameters: ";

    params += ("deviceIp: " + deviceIp.getHostAddress());
    params += (", forceNoBackup: " + forceNoBackup);
    params += (", factoryDefault: " + factoryDefault);
    params += (", restoreFromBackup: " + restoreFromBackup);
    params += (", backupFolder: " + backupFolder);
    params += (", skipPresetsAtRestore: " + skipPresetsAtRestore);
    params += (", updateVs100: " + updateVs100);
    params += (", uploadDefaultMiniWeb: " + uploadDefaultMiniWeb);

    logger.debug(params);
}

function uploadFiles(partNumber, parentCheckpoint, checkpointWeight) {
    var checkpoint = parentCheckpoint.add("Uploading and verifying files...", checkpointWeight);

    /* Add Firmwares */
    var filesToUpload = [APPLICATION_FILE];
    if (((PN_MMX4x2_HT200 === partNumber) || (PN_HDMI_TPS_RX200_VER_ENG === partNumber)) && updateVs100) {
        filesToUpload.push(VS100RX_FILE);

        var vsSpiSel = new GpioFile(deviceIp, 0, VS_SPI_SEL_GPIO);
        vsSpiSel.setLow();

        var vsResetN = new GpioFile(deviceIp, 1, VS_RESETN_GPIO);
        vsResetN.setLow();
    }

    /* Upload files */
    for (var i = 0; i < filesToUpload.length; i++) {
        uploadFileWithTftp(filesToUpload[i], checkpoint, 1.0 / filesToUpload.length);
    }
}

