/******************************************************************************
 * Declaration of UID (FVS) Eeprom
 *****************************************************************************/
 
/* create UID EEPROM/SDA GpioCnfig */
var SDA_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_11)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/* create UID EEPROM/SCL GpioCnfig */
var SCL_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_10)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/* define UID Eeprom i2c address */
var UID_EEPROM_I2C_ADDRESS = 0xA0;

/* create UID Eeprom I2C Eeprom config */
var UID_FVS_EEPROM_CONFIG = I2cEepromConfig.newBuilder()
    .setScl(SCL_UID)
    .setSda(SDA_UID)
    .setI2cAddress(UID_EEPROM_I2C_ADDRESS)
    .setEepromType(I2cEepromConfig.EepromType.EEPROM_24LC512K)
    .build();

/* create UID Eeprom Write Protect GpioConfig */
var UID_WP_GPIO = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOD)
    .setPin(GpioConfig.GPIO_PIN_7)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/******************************************************************************
 * Declaration of the SPI Flash for VS100
 *****************************************************************************/

 /* create SPI Chip Select~ GpioConfig */
var VS_SPI_CSN = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_4)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI SCLK GpioConfig */
var VS_SPI_SCLK = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_2)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI MOSI GpioConfig */
var VS_SPI_MOSI = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_6)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI MISO GpioConfig */
var VS_SPI_MISO = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_5)
    .setGpioMode(GpioConfig.GpioConfig_MODE_INPUT)
    .build();

/* create VS100 Spi Flash Config */
var VS_SPI_FLASH_CONFIG = ExternalSpiFlashConfig.newBuilder()
    .setCsnPin(VS_SPI_CSN)
    .setSclkPin(VS_SPI_SCLK)
    .setMosiPin(VS_SPI_MOSI)
    .setMisoPin(VS_SPI_MISO)
    .setFlashType(ExternalSpiFlashConfig.FlashType.N25Q064)
    .build();

/* create VS100 SPI select GpioConfig */
var VS_SPI_SEL_GPIO = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOB)
    .setPin(GpioConfig.GPIO_PIN_9)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/* create VS100 reset GpioConfig */
var VS_RESETN_GPIO = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOF)
    .setPin(GpioConfig.GPIO_PIN_2)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/******************************************************************************
 * Declaration of the SPI Flash
 *****************************************************************************/
 
/* create SPI Chip Select~ GpioConfig */
var SPI_CSN = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOD)
    .setPin(GpioConfig.GPIO_PIN_0)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI SCLK GpioConfig */
var SPI_SCLK = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_10)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI MOSI GpioConfig */
var SPI_MOSI = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_12)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create SPI MISO GpioConfig */
var SPI_MISO = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_11)
    .setGpioMode(GpioConfig.GpioConfig_MODE_INPUT)
    .build();

/* create External Spi Flash Config */
var EXTERNAL_SPI_FLASH_CONFIG = ExternalSpiFlashConfig.newBuilder()
    .setCsnPin(SPI_CSN)
    .setSclkPin(SPI_SCLK)
    .setMosiPin(SPI_MOSI)
    .setMisoPin(SPI_MISO)
    .setFlashType(ExternalSpiFlashConfig.FlashType.N25Q064)
    .build();

/******************************************************************************
 * Declaration of the files
 *****************************************************************************/

/* create Application file (Internal Flash File) */
var APPLICATION_FILE = InternalFlashFile.newBuilder()
    .setDeviceIp(deviceIp)
    .setBaseAddress(0x0800C000)
    .setAccessMode(InternalFlashFile.FILEACCESS_TFTP_READWRITE)
    .setPackageFile(packageFile)
    .setFileNameInPackage("tps_rx200.bin")
    .setFvsBlockNumber(0)
    .setFvsEepromConfig(UID_FVS_EEPROM_CONFIG)
    .setDescription("Firmware of the MCU")
    .build();

var UPB_EXTENSION = InternalFlashFile.newBuilder() // TPSRX200-2841
    .setDeviceIp(deviceIp)
    .setBaseAddress(0x08008000)
    .setAccessMode(InternalFlashFile.FILEACCESS_TFTP_READWRITE)
    .setPackageFile(packageFile)
    .setFileNameInPackage("mmx4x2_upb_ext.bin")
    .setFvsBlockNumber(2)
    .setFvsEepromConfig(UID_FVS_EEPROM_CONFIG)
    .setDescription("UPB extension of MMX4x2-HDMI")
    .build();

/* create VS100 file (VS100 external Spi Flash File) */
var VS100RX_FILE = ExternalSpiFlashFile.newBuilder()
    .setDeviceIp(deviceIp)
    .setExternalSpiFlashConfig(VS_SPI_FLASH_CONFIG)
    .setBaseAddress(0x000000)
    .setPackageFile(packageFile)
    .setFileNameInPackage("vs100_rx_cp0_cs5.bin")
    .setFvsBlockNumber(1)
    .setFvsEepromConfig(UID_FVS_EEPROM_CONFIG)
    .setDescription("Firmware of the VS100")
    .build();
