/**
deletionScript
var SampleDeletionScript = [
    //removed KM layer
    {version: "1.1.0b0", regex: /\/MEDIA\/KM(.*)$/g },
    //removed /MEDIA/AUDIO/I1, /MEDIA/AUDIO/I2, /MEDIA/AUDIO/I3 and /MEDIA/AUDIO/I4 ports
    {version: "1.1.0b0", regex: /\/MEDIA\/AUDIO\/I[1-4](.*)$/g },
];
 */ 
function EventManagerConverter(parser, version, revision, partNumber, deletionScript) {
    parser.convertLines(function(line) {
        if(line.node.startsWith('/EVENTS/')) {
            if(line != null && line.node.startsWith('/EVENTS/') && ((line.prop=='Condition') || (line.prop=='Action'))) {
                if(line.value.startsWith("/")) {
                    if(deletionScript != null) {
                        for(var i = 0; i < deletionScript.length; i++){
                            if(isConversionNeeded(version, deletionScript[i].version)){
                                deletionScript[i].regex.lastIndex = 0; //JavaScript RegExp object is stateful, so need to reset it manually
                                var match = deletionScript[i].regex.exec(line.value);
                                if(match != null) {
                                    parser.warnings.push( {data:line.printWithoutCommand(), msg:"Can't convert event, it will be lost"});
                                    return null;
                                }
                            }
                        }
                    }
                    var eventmanagervariablepattern = /^(\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*):(.*)$/g;
                    var convertedline = line.value.replace(eventmanagervariablepattern, "$1.$2=$3");
                    var hasVar = (convertedline !== line.value);
                    var hasTriggeringEveryChange = false;
                    if(!hasVar) {
                        var eventmanagertriggeringeverychangepattern = /^(\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*)?$/g;
                        convertedline = line.value.replace(eventmanagertriggeringeverychangepattern, "$1.$2=");
                        hasTriggeringEveryChange = (convertedline !== line.value);
                    }
                    var ret = convert(convertedline, version, revision, partNumber );
                    var evmparser = new Parser(ret.restore);
                    if(hasVar) {
                        line.value = evmparser.lines[0].printAsVariable();
                    } else if(hasTriggeringEveryChange) {
                        line.value = evmparser.lines[0].printAsVariable().replace(":", "?");
                    } else {
                        line.value = evmparser.lines[0].printWithoutCommand(true);
                    }
                }
            }
        }
        return line;
    });
    return parser;
};

    
/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/* global LW3line */

//only run this, if the backup contains the EM v1
function EventManagerv1to2(parser, actualVersion, v2version) {
    if(!isConversionNeeded(actualVersion, v2version)) {
        return parser;
    }
    //collect events
    var events = new Array();
    parser.convertLines(function(line) {
        if(line.node.startsWith('/EVENTS/')) {
            if(events[line.node] == null) {
                events[line.node] = new Array();
            }
            events[line.node][line.prop] = line.value;
            return null;
        }
        return line;
    });
    for(var event in events) {
        parser.addLine(LW3line.method(event, "Reset", ""));
        var enabled = false;
        if(events[event].SourceNode && events[event].SourceProperty) {
            var value = LW3line.property(events[event].SourceNode, events[event].SourceProperty, events[event].SourcePropertyValue).printWithoutCommand(true);
            parser.addLine(LW3line.property(event, "Condition", value));
            enabled = true;
        }
        if(events[event].TargetNode && events[event].TargetProperty) {
            var value = LW3line.property(events[event].TargetNode, events[event].TargetProperty, events[event].TargetPropertyValue).printWithoutCommand(true);
            parser.addLine(LW3line.property(event, "Action", value));
            enabled = true;
        }
        if(enabled) {
            parser.addLine(LW3line.property(event, "Enabled", "true"));
        }
        if(events[event].Name) {
            parser.addLine(LW3line.property(event, "Name", events[event].Name));
        }
    }
    return parser;
};
function convertLegacyXP(parser) {
    parser.convertLines(function(line) {
        if(line.node == "/SYS/MB/LEGACYAUDIOXP") {
            line.node = "/MEDIA/AUDIO/XP";
            switch(line.prop) {
                case "EmbedAudioInputToHdmiOut1":
                if(line.value == "true") {
                    line.value = "A3:O1";
                } else {
                    line.value = "A1:O1";
                }
                break;
                case "EmbedAudioInputToHdmiOut2":
                if(line.value == "true") {
                    line.value = "A3:O2";
                } else {
                    line.value = "A2:O2";
                }
                break;
                case "EnableAnalogPassthrough":
                if(line.value == "true") {
                    line.value = "A3:O3";
                } else {
                    line.value = "A2:O3";
                }
                break;
            }
            line.prop = "switch";
            line.type = "meth";
        }
        return line;
    });
    return parser;
}

function convertAnalogMuteToXPMute(parser, input, output) {
    parser.convertLines(function(line) {
        if(line.prop == "Mute") {
            switch(line.node) {
                case "/SYS/MB/ADAUIN/ANALOGIN":
                case "/MEDIA/AUDIO/"+input:
                    line.node = "/MEDIA/AUDIO/XP";
                    if(line.value == "true") {
                        line.prop = "muteSource"
                    } else {
                        line.prop = "unmuteSource"
                    }
                    line.value = input;
                    line.type = "meth";
                break;

                case "/SYS/MB/ADAUOUT/ANALOGOUTDIFF":
                case "/SYS/MB/ADAUOUT/ANALOGOUTHP":
                case "/MEDIA/AUDIO/"+output:
                    line.node = "/MEDIA/AUDIO/XP";
                    if(line.value == "true") {
                        line.prop = "muteDestination"
                    } else {
                        line.prop = "unmuteDestination"
                    }
                    line.value = output;
                    line.type = "meth";
                break;
            }
        }
        return line;
    });
    return parser;
}

function convert(restoreData, version, revision, partNumber) {
    var partNumInt = Number(replaceAt(partNumber, 4, '0')); //mask th dvigear flag
    // Decide which script is to be used
    switch (partNumInt) {
        case 91540021: // HDMI-TPS-RX110AY
        case 91540070: // HDMI-TPS-RX110AY-Plus
        case 91540022: // HDMI-TPS-RX220AK
        case 91310034: // MMX4x2-HDMI
        case 91310035: // MMX4x2-HT200
        case 91540039: // HDMI-TPS-RX200_VER_ENG
        case 91310076: // MMX4x2-HDMI-USB20-L
        case 91310077: // MMX4x2-HDMI-USB20-Slim
            break;
        default:
            throw "Device is not supported";
    }

    var devicePresetHasMacros = DevicePresetHasMacros(restoreData);
    if(devicePresetHasMacros != null) {
        var ret = {
            restore: restoreData
        };
        return ret;
    }

    var parser = new Parser(restoreData);

    var MMX4x2ConversionScript = [
    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/ADAU/ANALOGIN",            null,                   null) , replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", null,       null)},
    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/ADAU/ANALOGIN",            "Volume",               null) , replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "VolumeDb", null)},
    {   version: "1.2.0b0", search: LW3line.property("/MEDIA/AUDIO/I5",                  null,                   null) , replace: LW3line.property("/MEDIA/AUDIO/A3",         null,       null)},
//    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/DEVICEUSB/USBDCONNECTOR",  null,                   null) , replace: null},
//    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/DEVICEUSB/USBDCONTROL",    null,                   null) , replace: null},

    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOIN3",            "Text",                 null) , replace: null},
    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOIN4",            "Text",                 null) , replace: null},
    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOIN5",            "Text",                 null) , replace: null},
    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/TPS/AUDIO",                "Text",                 null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT0",           "EmbedSource",          null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT0",           "SpdifMode",            null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT0",           "HdmiAudioOutputEn",    null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "EmbedSource",          null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "SpdifMode",            null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "HdmiAudioOutputEn",    null) , replace: null},


    ];
    var MMX4x2EMDeletionScript = [
        {version: "1.2.0b0", regex: /\/MANAGEMENT\/TASK(.*)$/g },
        {version: "1.2.0b0", regex: /\/SYS\/MB\/DEVICEUSB(.*)$/g },
        {version: "1.2.0b0", regex: /\/MEDIA\/AUDIO\/I[1-4](.*)$/g },
        {version: "1.2.0b0", regex: /\/SYS\/MB\/TPS\/AUDIO(.*)$/g },
        {version: "1.2.0b0", regex: /\/SYS\/MB\/HDMI\/AUDIOIN[3-5](.*)$/g },
        {version: "1.3.1b0", regex: /\/SYS\/MB\/HDMI\/O[1-2]\.EmbedSource(.*)$/g },
        {version: "1.3.1b0", regex: /\/SYS\/MB\/HDMI\/O[1-2]\.SpdifMode(.*)$/g },
        {version: "1.3.1b0", regex: /\/SYS\/MB\/HDMI\/O[1-2]\.HdmiAudioOutputEn(.*)$/g },
    ];

    var TPSRX110AYConversionScript = [
    {   version: "1.2.0b0", search: LW3line.property(null,                               "deEmbeddingStatus",    null) , replace: LW3line.property(null, "DeEmbeddingStatus", null)},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "EmbedSource",          null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "SpdifMode",            null) , replace: null},
    {   version: "1.3.1b0", search: LW3line.property("/SYS/MB/HDMI/AUDIOOUT1",           "HdmiAudioOutputEn",    null) , replace: null},
//    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/DEVICEUSB/USBDCONNECTOR",  null,                   null) , replace: null},
//    {   version: "1.2.0b0", search: LW3line.property("/SYS/MB/DEVICEUSB/USBDCONTROL",    null,                   null) , replace: null},
    ];
    
    var TPSRX110AYEMDeletionScript = [
        {version: "1.2.0b0", regex: /\/MANAGEMENT\/TASK(.*)$/g },
        {version: "1.2.0b0", regex: /\/MEDIA\/KM(.*)$/g },
        {version: "1.2.0b0", regex: /\/SYS\/MB\/DEVICEUSB(.*)$/g },
    ];

    var TPSRX220AKConversionScript = [
    ];

    var MMX4x2HDMIUSB20LConversionScript = [
    ];

    switch (partNumInt) {
        case 91310034: // MMX4x2-HDMI
            MMX4x2EMDeletionScript.push({version: "1.2.0b0", regex: /\/SYS\/MB\/ETH\/CPU(.*)$/g });
            MMX4x2EMDeletionScript.push({version: "1.2.0b0", regex: /\/MEDIA\/ETHERNET\/P2(.*)$/g });
            MMX4x2ConversionScript.push({version: "1.2.0b0", search: LW3line.property("/SYS/MB/ETH/CPU",null,null) , replace: null});
        case 91310035: // MMX4x2-HT200
        case 91540039: // HDMI-TPS-RX200_VER_ENG
            SimpleScriptConverter(parser, version, MMX4x2ConversionScript);
            if (isConversionNeeded(version, "1.2.0b0")) {
                convertLegacyXP(parser);
                convertAnalogMuteToXPMute(parser, "A3", "O3"); //A3, because MMX4x2ConversionScript already converted it.
            }
            EventManagerConverter(parser, version, revision, partNumber, MMX4x2EMDeletionScript);
            break;
        case 91540021: // HDMI-TPS-RX110AY
        case 91540070: // HDMI-TPS-RX110AY-Plus
            SimpleScriptConverter(parser, version, TPSRX110AYConversionScript);
            if (isConversionNeeded(version, "1.2.0b0")) {
                convertAnalogMuteToXPMute(parser, null, "O2");
            }
            EventManagerConverter(parser, version, revision, partNumber, TPSRX110AYEMDeletionScript);
            break;
        case 91540022: // HDMI-TPS-RX220-AK
            SimpleScriptConverter(parser, version, TPSRX220AKConversionScript);
            EventManagerConverter(parser, version, revision, partNumber);
            break;
        case 91310076: // MMX4x2-HDMI-USB20-L
        case 91310077: // MMX4x2-HDMI-USB20-Slim
            SimpleScriptConverter(parser, version, MMX4x2HDMIUSB20LConversionScript);
            EventManagerConverter(parser, version, revision, partNumber);
            break;
        default:
            throw "Device is not supported";
    }

    parser = SimpleSorter(parser, new LW3line("/MANAGEMENT/NETWORK", null, null));

    return parser.output();
};


function LW3line(node, prop, value, type) {
    if(node === undefined) {
        this.node=null;
    } else {
        this.node=node;
    }
    if(prop === undefined) {
        this.prop=null;
    } else {
        this.prop=prop;
    }
    if(value === undefined) {
        this.value=null;
    } else {
        this.value=value;
    }
    if(type === undefined) {
        this.type=null;
    } else {
        this.type=type;
    }
    this.isProperty = function() {
        return this.type === "prop";
    };
    
    this.isMethod = function() {
        return this.type === "meth";
    };
    
    this.print = function(isProp) {
        if(isProp === undefined) {
            isProp = this.isProperty();
        }
        if(isProp) {
            return 'SET ' + this.printWithoutCommand(isProp);
        } else {
            return 'CALL ' + this.printWithoutCommand(isProp);
        }
    };
    this.printWithoutCommand = function(isProp) {
        if(isProp === undefined) {
            isProp = this.isProperty();
        }
        if(isProp) {
            return this.node + '.' + this.prop + '=' + this.value;
        } else {
            return this.node + ':' + this.prop + '(' + this.value + ')';
        }
    };
    this.printAsVariable = function() {
        return this.node + '.' + this.prop + ':' + this.value;
    };
};

LW3line.property = function(node, prop, value) {
    return new LW3line(node, prop, value, "prop");
};

LW3line.method = function(node, prop, value) {
    return new LW3line(node, prop, value, "meth");
};

LW3line.parse = function(line) {
    var prop = /^SET (\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*)=(.*)$/g;
    var meth = /^CALL (\/[a-zA-Z0-9\/]*)\:([a-zA-Z0-9]*)\((.*)\)$/g;
    var eventmanagercompatibility = /^(\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*)=(.*)$/g;
    var match = prop.exec(line);
    if(match != null) {
        return LW3line.property(match[1], match[2], match[3]);
    }
    match = meth.exec(line);
    if(match != null) {
        return LW3line.method(match[1], match[2], match[3]);
    }
    match = eventmanagercompatibility.exec(line);
    if(match != null) {
        return new LW3line(match[1], match[2], match[3]);
    }
    return null;
};

function Parser(data) {
    var output = "";
    this.lines = new Array();
    this.warnings = new Array();
    data = data.split('\r').join('\n'); 
    var tmp = data.split('\n');
    for(var lineIndex in tmp) {
        var line = tmp[lineIndex];
        if (line.length > 0) {
            if (!line.startsWith(';')) {
                var m = LW3line.parse(line);
                if(m != null) {
                    this.lines.push(m);
                } else {
                    throw "Parse error: " + line;
                }
            } else {
                output = output + line + "\r\n"; 
            }
        }
    }
    
    this.convertLines = function(convertFunc) {
        var newLines = new Array();
        for(var line in this.lines) {
            var newLine = convertFunc(this.lines[line], this);
            if(newLine != null) {
                newLines.push(newLine);
            }
        }
        this.lines = newLines;
    };
    this.addLine = function(line) {
        this.lines.push(line);
    };
    this.putWarning = function(line, warn) {
        warnings.push({data:line.print(), msg:warn});
    };
    this.printLines = function() {
        for(var lineNum in this.lines) {
            output = output + this.lines[lineNum].print() + "\r\n"; 
        }
        return output;
    };
    this.output = function() {
        var ret = {
            restore: this.printLines()
        };
        if(this.warnings.length>0) {
            ret.warning = this.printWarnings();
        }
        return ret;
    };
    this.printWarnings = function() {
        var output = "";
        for(var lineNum in this.warnings) {
            output = output + this.warnings[lineNum].data + ": " + this.warnings[lineNum].msg + "\r\n"; 
        }
        return output;
    };
};/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
// This function converts the version string into a number to make it comparable
// According to the LWR_UID driver, versions are like:
// [uint8_t].[uint8_t].[uint8_t]b[uint8_t]
function versionToNumber(versionString){
    var pattern = /(\d+)\.(\d+)\.(\d+)b(\d+)/;
    var numbers = pattern.exec(versionString);
    if(numbers == null){
        throw "ERROR: Wrong version number in converter script";
    }
    
    var ret  = Number(numbers[1] * 256 * 256 * 256);
    ret     += Number(numbers[2] * 256 * 256);
    ret     += Number(numbers[3] * 256);
    ret     += Number(numbers[3]);
    return ret;
}

function isConversionNeeded(actualVersion, convertedVersion) {
    return versionToNumber(actualVersion) < versionToNumber(convertedVersion);
}

function replaceAt(string, index, character) {
    return string.substr(0, index) + character + string.substr(index+character.length);
};

function importJs(file) {
    $.getScript(file, function(){
        alert(file + "Script loaded but not necessarily executed.");
    });
};

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

// WARNING: version numbers must be in ascending order, the code won't check it for you!
/*
 * in search null means don't care
 * in replace null means don't change
var SampleConversionScript = [
    //remove node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: null},
    //move node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},

    //remove property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: null},
    //move property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", "ControlLock", null)},
];
 */
function SimpleScriptConvert(convert) {
    this.paramPatern = /\{([\d]+)\}/g;
    
    this.parseParam = function(search, replace, pattern) {
        var ret = Array();
        if(search == null) {
            ret.search_matches = ["{0}"];
            ret.search = new RegExp(pattern + "$", 'g');
        } else {
            this.paramPatern.lastIndex = 0;
            ret.search_matches = search.match(this.paramPatern);
            if(ret.search_matches == null) {
                ret.search_matches = ["{0}"];
            } else {
                ret.search_matches.unshift("{0}");
            }
            this.paramPatern.lastIndex = 0;
            ret.search = new RegExp(search.replace(this.paramPatern, pattern) + "$", 'g');
        }
        ret.replace = replace;
        return ret;
    };
    
    this.convertParam = function(matches, params) {
        if(params.replace == null) {
            return null;
        }
        var ret = params.replace;
        if(matches.length > 0) {
            for(var i = 0; i < matches.length; i++) {
                ret = ret.replace( params.search_matches[i], matches[i]);
            }
        }
        return ret;
    };
    this.getParamMatch = function(params, original) {
        if(original == null) {  //EM compatibility
            return new Array();
        }
        params.search.lastIndex = 0;
        if(!params.search.test(original)) {
            return null;
        }
        params.search.lastIndex = 0;
        var match = params.search.exec(original);
        if(match == null) {
            match = new Array();
        }
        match[0] = original;
        return match;
    };
    
    this.convertLine = function(line) {
        var node = this.getParamMatch(this.node, line.node);
        if(node == null) {
            return line;
        }
        var prop = this.getParamMatch(this.prop, line.prop);
        if(prop == null) {
            return line;
        }
        var value = this.getParamMatch(this.value, line.value);
        if(value == null) {
            return line;
        }
        var type = this.getParamMatch(this.type, line.type);
        if(type == null) {
            return line;
        }
        
        line.node = this.convertParam(node, this.node);
        line.prop = this.convertParam(prop, this.prop);
        line.value = this.convertParam(value, this.value);
        line.type = this.convertParam(type, this.type);
        if(line.node == null || line.prop == null || line.value == null || line.type == null) {
            return null;
        } else {
            return line;
        }
    };
    
    this.replaceReplacer = function(param) {
        return (param==null)?"{0}":param;
    };
    
    var replace;
    if(convert.replace != null) {
        replace =  {
            node:this.replaceReplacer(convert.replace.node),
            prop:this.replaceReplacer(convert.replace.prop),
            value:this.replaceReplacer(convert.replace.value),
            type:this.replaceReplacer(convert.replace.type)
        };
    } else {
        replace = {
            node:null, 
            prop:null, 
            value:null, 
            type:null
        };
    }
    this.node = this.parseParam(convert.search.node, replace.node, "([a-zA-Z0-9\\\/]*)");
    this.prop = this.parseParam(convert.search.prop, replace.prop, "([a-zA-Z0-9]*)");
    this.value = this.parseParam(convert.search.value, replace.value, "(.*)");
    this.type = this.parseParam(convert.search.type, replace.type, "(.*)");
};


function SimpleScriptConverter(parser, version, conversionScript){
    // Find the first version number to be used
    var firstPosition = -1;
    var actualVersionNumber = versionToNumber(version);
    for(var i = 0; i < conversionScript.length; i++){
        if(versionToNumber(conversionScript[i].version) > actualVersionNumber){
            firstPosition = i;
            break;
        }
    }
    // All versions in the script are older than the current version, 
    // no conversion is needed
    if(firstPosition === -1)
        return parser;
    
    // Do the conversion line-by-line
    for(var scriptIndex = firstPosition; scriptIndex < conversionScript.length; scriptIndex++){
        var converter = new SimpleScriptConvert(conversionScript[scriptIndex]);
//        parser.convertLines(converter.convertLine);
        parser.convertLines(function(line) {
            return converter.convertLine(line);
        });
    }
    
    return parser;
}

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

// WARNING: version numbers must be in ascending order, the code won't check it for you!
/*
 * in search null means don't care
 * in replace null means don't change
var SampleConversionScript = [
    //remove node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: null},
    //move node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},

    //remove property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: null},
    //move property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", "ControlLock", null)},
];
 */

function SimpleSorter(parser, moveToEnd){
    
    var lines = new Array();
    parser.convertLines(function(line) {
        if(SimpleSorter.isMatching(line, moveToEnd)) {
            lines.push(line);
            return null;
        }
        return line;
    });
    for(var line in lines) {
        parser.addLine(lines[line]);
    }
    return parser;
}
SimpleSorter.isPatternMatching = function(value, pattern) {
    return ((value == null) || (pattern == null) || (value == pattern));
};

SimpleSorter.isMatching = function(line, filter) {
    return SimpleSorter.isPatternMatching(line.type, filter.type) && SimpleSorter.isPatternMatching(line.node, filter.node) && SimpleSorter.isPatternMatching(line.prop, filter.prop) && SimpleSorter.isPatternMatching(line.value, filter.value);
};
function DevicePresetHasMacros(restoreData) {
    var macroRegex = /^\;Begin.*/gm;
    return macroRegex.exec(restoreData);
};
/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/*
var TPSLayerConversion = [
    {layer: "VIDEO", inputNum: 2, outputNum: 2, inputString: "I", outputString: "O"},
    {layer: "AUDIO", inputNum: 2, outputNum: 2, inputString: "I", outputString: "O"},
];
 */

function Crosspointv1to2_layer(parser, converter) {
    parser.convertLines(function(line) {
        if(line.node.startsWith("/MEDIA/"+ converter.layer)) {
            if(line.node.search("XP") !== -1) {
                line.node="/MEDIA/"+ converter.layer + "/XP";
                if(line.prop === "setSourcePortPriority") {
                    line.prop = "setAutoselectionPriority";
                    var pattern = /P(\d):(\d)/;
                    var parameters = line.value.split(";");
                    var value = "";
                    // parameters = [P0:1, P2:1, ...]
                    for(var outputIndex = 1; outputIndex <= converter.outputNum; outputIndex++){
                        for(var j in parameters){
                            if(Number(j) !== 0 || outputIndex !== 1)
                                value += ";"
                            var x = pattern.exec(parameters[j]);
                            var portNum = x[1];
                            var prio = x[2];
                            value += converter.inputString + portNum + "\\(" + converter.outputString + outputIndex + "\\):" + prio;
                        }
                    }
                    line.value = value;
                } else if(line.prop === "setDestinationPortAutoselect") {
                    var ports = line.value.split(";");
                    var value = "";
                    for(var k in ports){
                        // ports[k] = P5:DPB
                        if(Number(k) !== 0)
                            value += ";";
                        var x = /P(\d):(.+)/.exec(ports[k]);
                        var portNum = Number(x[1]);
                        var text = x[2];
                        value += converter.outputString + (portNum - converter.inputNum) + ":";
                        if(text.charAt(0) === "D")
                            value += "D";
                        else
                            value += text.charAt(0) + text.charAt(1);
                    }
                    line.value = value;
                }
            } else { //check if its port, need to convert from P
                var match = /([A-Z\/]+)P(\d)$/g.exec(line.node);
                if(match != null) {
                    var portNum = Number(match[2]);
                    if(portNum <= converter.inputNum) { //if its input
                        line.node = match[1] + converter.inputString + portNum;
                    } else { //if its output
                        line.node = match[1] + converter.outputString + (portNum - converter.inputNum);
                    }
                }
            }
        }
        return line;
    });
    return parser;
};
function Crosspointv1to2(parser, layerConversion, actualVersion, v2version){
    if(!isConversionNeeded(actualVersion, v2version)) {
        return parser;
    }
    for(var converter in layerConversion) {
        parser = Crosspointv1to2_layer(parser, layerConversion[converter]);
    }
    return parser;
};


function TPSconvertv110(parser, version, deviceDescriptor ) {

        // Do EventManager conversion
        parser = EventManagerv1to2(parser, version, "1.1.0b0");

        // Do crosspoint settings conversion
        var TPSLayerConversion = [
            {layer:"VIDEO", inputNum: deviceDescriptor.inputVideoNum, outputNum: deviceDescriptor.outputVideoNum, inputString: "I", outputString: "O"},
            {layer:"AUDIO", inputNum: deviceDescriptor.inputAudioNum, outputNum: deviceDescriptor.outputAudioNum, inputString: "I", outputString: "O"}
        ];

        parser = Crosspointv1to2(parser, TPSLayerConversion, version, "1.1.0b0");

        // Do simple find-and-replace conversion
        var TPSConversionScriptv110 = [
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB", "ButtonLock", null) , replace: LW3line.property("/MANAGEMENT/UI", "ControlLock", null)},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "tpsModeSetting", null) , replace: LW3line.property("/SYS/MB/TPS/REMOTE", "tpsModeSetting", null)},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "pwr5vMode", null) , replace: LW3line.property("/SYS/MB/TPS/VS100OUT", "Pwr5vMode", null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/ETH/LINK", null, null) , replace: new LW3line("/SYS/MB/TPS/ETH", null, null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/RS232/LINK", null, null) , replace: new LW3line("/SYS/MB/TPS/RS232", null, null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/CECIR", null, null) , replace: null},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/HDMI/LOCALOUT", "PreviewEnable", null) , replace: null},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "PreviewEnable", null) , replace: null},
            {version: "1.1.0b0", search: LW3line.method(null, "SendMessage", null) , replace: LW3line.method(null, "sendMessage", null)},
            {version: "1.1.0b0", search: LW3line.method(null, "SendText", null) , replace: LW3line.method(null, "sendText", null)},
            {version: "1.1.0b0", search: LW3line.method(null, "SendBinaryMessage", null) , replace: LW3line.method(null, "sendBinaryMessage", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "LinkState", null) , replace: LW3line.property("/REMOTE/D1", "LinkState", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "tpsMode", null) , replace: LW3line.property("/REMOTE/D1", "tpsMode", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "connectedDevice", null) , replace: LW3line.property("/REMOTE/D1", "ConnectedDevice", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "DeviceId", null) , replace: LW3line.property("/REMOTE/D1", "DeviceId", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "VendorId", null) , replace: LW3line.property("/REMOTE/D1", "VendorId", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "tpsModeSetting", null) , replace: LW3line.property("/REMOTE/D1", "tpsModeSetting", null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/UMXDP{1}", null, null) , replace: new LW3line("/SYS/MB/DP{1}", null, null)},
        ];
        parser = SimpleScriptConverter(parser, version, TPSConversionScriptv110);
        
        return parser;
};

